require "asm/provider/switch/base"

module ASM
  class Provider
    class Switch
      # A provider to configure Force10 switches
      #
      # Force10 switches are broken down into many different models with different puppet
      # modules and completely different means of configuring vlans and interfaces between
      # them.
      #
      # To manage the complexity resulting from the wide range of Force10 switches this class
      # use something called creators to take care of interacting with the switches. See the
      # {ASM::Provider::Switch::Force10::Rack} and {ASM::Provider::Switch::Force10::Blade} classes
      # for examples
      class Force10 < Switch::Base
        puppet_type "force10"

        def self.handles_switch?(switch)
          !!(switch["refId"] =~ /^(dell_ftos|dell_iom)/)
        end

        def normalize_facts!
          super

          if facts["interfaces"].is_a?(Array)
            facts["interfaces"] = facts["interfaces"].map do |interface|
              # Some force10 switches like the S5000 has an interface fact showing all
              # tagged and untagged vlans etc and not just the list of interface names
              # this parses those cases if they appear to be a string matching that
              if interface.is_a?(String) && interface =~ /untagged_vlans/
                JSON.parse(interface)
              else
                interface
              end
            end
          end
        end

        def server_supported?(server)
          return true if (server.rack_server? || server.tower_server?) && rack_switch?

          return true if server.blade_server? && blade_switch?
          false
        end

        def load_switch_creator(creator)
          path = "asm/provider/switch/force10/%s" % creator
          logger.debug("Configuring %s using %s" % [type.puppet_certname, path])
          require path
        end

        def resource_creator!
          if rack_switch?
            load_switch_creator("rack")
            self.resource_creator = Rack.new(self)
          elsif blade_ioa_switch?
            load_switch_creator("blade_ioa")
            self.resource_creator = BladeIoa.new(self)
          elsif blade_mxl_switch?
            load_switch_creator("blade_mxl")
            self.resource_creator = BladeMxl.new(self)
          else
            raise("Do not know how to manage resources for switch %s with model %s, no suitable resource creator could be found" % [type.puppet_certname, model])
          end
        end

        def initialize_ports!
          resource_creator.initialize_ports!
        end

        def resource_creator
          @resource_creator ||= resource_creator!
        end

        def additional_resources
          resource_creator.to_puppet
        end

        # Creates a ioa_interface resource
        #
        # @note this should only be used in cases where the {#Base#configure_interface_fvlan} approach isn't used
        # @param interface [String] the interface name
        # @param options [Hash]
        # @option options [Array<String>] :tagged_vlans list of VLANs tagged on this interface
        # @option options [Array<String>] :untagged_vlans list of VLANs untagged on this interface
        # @option options [Integer,nil] :portchannel portchannel this interface belongs to
        # @return [void]
        def ioa_interface_resource(interface, options={})
          options = {:tagged_vlans => [], :untagged_vlans => [], :portchannel => nil}.merge(options)

          resource_creator.ioa_interface_resource(interface, options)
        end

        # Creates a port channel resource
        #
        # @param number [String,Fixnum] the port channel number to create
        # @param options [Hash]
        # @option options [Boolean] :edge_port sets edge_port spanning tree settings when true
        # @option options [Boolean] :fcoe enables fip_snooping_fcf when true
        # @option options [Boolean] :remove sets ensure = absent when true
        # @option options [Boolean] :vlt_peer sets true = false when true it will configure vlt-peer-lag in switch
        # @option options [Array] :tagged list of tagged vlans to set for the portchannel
        # @option options [Array] :untagged list of tagged vlans to set for the portchannel
        # @return [void]
        # @raise [StandardError] when attempting to manage the same resource twice
        # @raise [StandardError] when attempting to manage unsupported switches
        def portchannel_resource(number, options={})
          resource_creator.portchannel_resource(number, options)
        end

        # Creates a mxl vlan resource
        #
        # @param vlan [String,Fixnum] the vlan number to create
        # @param name [String] the vlan name
        # @param description [String] the vlan description
        # @param port_channels [Array<String>] list of tagged port channels
        # @param remove [Boolean] sets ensure = absent when true
        # @return [void]
        # @raise [StandardError] when attempting to manage the same resource twice
        # @raise [StandardError] when attempting to manage unsupported switches
        def mxl_vlan_resource(vlan, name, description, port_channels, remove=false)
          resource_creator.mxl_vlan_resource(vlan, name, description, port_channels, remove)
        end

        # Creates a MXL interface resource
        #
        # @param interface [String] the interface to manage
        # @param port_channel [String,Fixnum] the port channel it belongs to
        # @return [void]
        # @raise [StandardError] when attempting to manage the same resource twice
        # @raise [StandardError] when attempting to manage unsupported switches
        def mxl_interface_resource(interface, port_channel=nil)
          resource_creator.mxl_interface_resource(interface.gsub("TenGigabitEthernet", "Te"), port_channel)
        end

        # Configures a list of interfaces for quodmode
        #
        # Only 40 Gb interfaces can be part of a group, any non 40 Gb interfaces
        # listed when enable is true will be silently skipped and logged
        #
        # @param interfaces [Array,nil] list of interfaces, when nil will use the quad_port_interfaces fact
        # @param enable [Boolean] should it be enabled or not for quadmode
        # @param reboot [Boolean] should the last interface set reboot required
        # @return [String] the last resource made that can be used to sequence puppet requires on
        def configure_quadmode(interfaces, enable, reboot=true)
          resource_creator.configure_quadmode(interfaces, enable, reboot)
        end

        # Configures the switch for VLT or PMUX mode
        #
        # @param pmux [boolean] configure pmux mode
        # @param ethernet_mode [Boolean] for use in PMUX configurations
        # @param vlt_data [Hash] by default nil normally a hash of vlt settings
        # @return [void]
        # @raise [StandardError] when called multiple times
        def configure_iom_mode!(pmux, ethernet_mode, vlt_data=nil)
          resource_creator.configure_iom_mode!(pmux, ethernet_mode, vlt_data)

          process!(:skip_prepare => true)
        end

        # Configures the switch using the force10_settings hash found in the component
        #
        # @note this will reset the creator and run puppet, be careful when running this in something that should be a sequence
        # @param settings [Hash] the force10_settings hash from a component configuration
        # @return [void]
        # @raise [StandardError] when invalid settings are received
        def configure_force10_settings(settings)
          if settings.nil? || !settings.is_a?(Hash) || settings.empty?
            raise("Received invalid force10_settings for %s: %s" % [type.puppet_certname, settings.inspect])
          end

          resource_creator!

          resource_creator.configure_force10_settings(settings)

          process!(:skip_prepare => true)
        end

        # Configures each partition as per the template
        #
        # @param server [ASM::Type::Server]
        # @return [void]
        def provision_server_networking(server)
          server.configured_interfaces.each do |interface|
            provision_server_interface(server, interface)
          end
        end

        # Resets associated switch ports to defaults
        #
        # When resetting the ports they are configured to default VLANs
        # by setting tagged and untagged vlans to '1'
        #
        # @note depending on how asm::force10 teardown is built this might need to change to become creator specific
        # @param server [ASM::Type::Server]
        # @return [void]
        def teardown_server_networking(server)
          server.configured_interfaces.each do |interface|
            partition = interface.partitions.first

            next unless port = type.find_mac(partition["mac_address"], :server => server)

            logger.info("Resetting port %s / %s to untagged vlan 1 and no tagged vlans" % [server.puppet_certname, partition.fqdd])

            resource_creator.configure_interface_vlan(port, "1", false, true, find_portchannel(server, interface))
          end
        end

        # (see ASM::Type::Server#configure_server)
        def configure_server(server, staged=false)
          if !server.teardown?
            provision_server_networking(server)
          else
            teardown_server_networking(server)
          end

          process! unless staged
        end

        def rack_switch?
          refId.start_with?("dell_ftos")
        end

        def blade_switch?
          blade_ioa_switch? || blade_mxl_switch? ? true : false
        end

        def blade_ioa_switch?
          !!model.match(/Aggregator|IOA|PE-FN/)
        end

        def blade_mxl_switch?
          !!model.match(/MXL/)
        end

        def fcflexiom_switch?
          return false unless blade_switch?

          !facts["interfaces"].grep(/fc/).empty?
        end

        def san_switch?
          facts["switch_fc_mode"] == "Fabric-Services"
        end

        def npiv_switch?
          facts["switch_fc_mode"] == "NPG"
        end

        def find_portchannel(server, interface)
          return nil unless use_portchannel?(server, interface)

          partition = interface.partitions.first
          interface_mac = partition.mac_address

          existing_reservation = portchannel_reservations.find {|_, macs| macs.include?(interface_mac)}
          return existing_reservation.first if existing_reservation

          team_info = team_switch_info(interface_mac, server)
          raise("No teaming information available for %s on %s" % [server.puppet_certname, interface_mac]) if team_info.empty?

          available_on_all = nil
          team_info.each do |_, info|
            available_on_all ||= info[:available_portchannels]
            available_on_all &= info[:available_portchannels]
          end
          portchannel = available_on_all.first

          raise("No portchannels available for LACP config across all switches") unless portchannel

          team_info.each do |mac, info|
            info[:switch].provider.reserve_portchannel!(portchannel, mac)
          end

          portchannel
        end

        # Collects port/switch/available portchannel info for all NICs in a team
        #
        # This information is needed to be collected for the team because best
        # practices for teamed portchannels are that the portchannel number
        # matches across the team on the different switches, so we want to find
        # one that all switches connected to the NICS in this team have
        # available.  This information will be used with {#reserve_portchannel!}
        # on the NICs' respective providers to make sure the portchannels match
        # across the team
        #
        # @param interface_mac [String] the interface's mac, used to find the team
        # @param server [ASM::Type::Server] the server object for the team
        # @return [Hash]
        def team_switch_info(interface_mac, server)
          team_switch_info = {}

          team = server.network_config.teams.find do |t|
            t[:mac_addresses].include?(interface_mac)
          end

          team[:mac_addresses].each do |mac|
            switch = type.switch_collection.switch_for_mac(mac, :server => server)
            port = switch.find_mac(mac, :server => server).gsub("Te ", "TenGigabitEthernet ")
            possible_portchannels = switch.provider.available_portchannels(port)
            team_switch_info[mac] = {:switch => switch, :port => port, :available_portchannels => possible_portchannels}
          end

          team_switch_info
        end

        # Returns a list of the available portchannels for a port
        #
        # If a port is already in a portchannel that can be reused, we will try
        # to prioritize that one by returning it first in the list
        #
        # @param port [String] the interface name
        # @return [Array]
        def available_portchannels(port)
          available = (1..128).find_all {|n| !portchannel_members[n.to_s] && !portchannel_reservations[n.to_s]}
          available.map!(&:to_s)

          # See if one of the used portchannels is for this port and can be reused
          used = portchannel_members.find_all do |_, members|
            # TODO: Multiple ports might be in the same switch and in the same
            # team, so they might have to share portchannel
            members.include?(port) && members.size == 1
          end.flatten.first

          # if there's a reusable portchannel for this port, prioritize it
          available.unshift(used) if used

          available
        end
      end
    end
  end
end
