require "asm/provider/switch/force10/base"

module ASM
  class Provider
    class Switch
      class Force10
        # Manage Interface VLAN membership for Dell Force10 blade switches
        #
        # These switches unlike their TOR cousins are configured using a simple
        # 1 resource per interface method so this class mainly builds a temporary
        # map of every interface and all its VLAN memberships via {#configure_interface_vlan}
        # and then turns that into an appropriate puppet resource using {#to_puppet}
        class BladeIoa < Base
          def iom_mode
            provider.facts["iom_mode"]
          end

          # Determines if the switch is in SMUX mode
          #
          # This seems pointless but the term 'smux' is never in the code
          # but people talk of smux switches, this helps greps.
          #
          # @return [Boolean]
          def smux?
            iom_mode == "standalone"
          end

          # (see Provider::Switch::Force10#portchannel_resource)
          def portchannel_resource(number, options={})
            # If we have an FNIOA in full-switch mode, it will be configured like
            # an MXL, so we use the force10_portchannel resource instead
            if iom_mode == "full-switch"
              super
              return
            elsif smux?
              logger.debug("Switch %s is in SMUX mode, not adding any port channels" % type.puppet_certname)
              return
            end

            number = number.to_s
            tagged = Array(options[:tagged])
            untagged = Array(options[:untagged])

            if has_resource?("ioa_portchannel", number)
              raise("Ioa_portchannel[%s] is already being managed on %s" % [number, type.puppet_certname])
            end

            resource = {
              "switchport" => "true",
              "portmode" => "hybrid",
              "shutdown" => "false",
              "ungroup" => "true",
              "mtu" => provider.mtu,
              "vltpeer" => "true",
              "ensure" =>  options[:action] == :remove ? "absent" : "present"
            }

            resource["tagged_vlan"] = tagged unless tagged.empty?
            resource["untagged_vlan"] = untagged unless untagged.empty?

            resource["require"] = sequence if sequence

            port_resources["ioa_portchannel"] ||= {}
            port_resources["ioa_portchannel"][number] = resource

            @sequence = "Ioa_portchannel[%s]" % number
            nil
          end

          def configure_iom_mode!(pmux, ethernet_mode, vlt_data=nil)
            raise("iom mode resource is already created in this instance for switch %s" % type.puppet_certname) if port_resources["ioa_mode"]

            if vlt_data
              interface = vlt_data["portMembers"]
              port_channel = vlt_data["portChannel"]
              destination_ip = vlt_data["Destination_ip"]
              unit_id = vlt_data["unit-id"]

              if vlt_data["model"] =~ /PE-FN.*/
                mode = "fullswitch"
              else
                mode = "vlt"
              end

              port_resources["ioa_mode"] = vlt_ioa_mode_resource(interface, port_channel, destination_ip, unit_id, mode)
            elsif pmux

              if type.model =~ /PE-FN/
                mode = "fullswitch"
              else
                mode = "pmux"
              end

              port_resources["ioa_mode"] = pmux_ioa_mode_resource(ethernet_mode, mode)
            end

            if port_resources["ioa_mode"]
              key, resource = port_resources["ioa_mode"].first
              resource["require"] = sequence if sequence
              @sequence = "Ioa_mode[%s]" % key
            end

            nil
          end

          # (see Provider::Switch::Force10#ioa_interface_resource)
          def ioa_interface_resource(interface, options={})
            if has_resource?("ioa_interface", interface)
              raise("Ioa_interface[%s] is already being managed on %s" % [interface, type.puppet_certname])
            end

            portchannel = options.fetch(:portchannel, nil)
            tagged_vlans = options.fetch(:tagged_vlans, [])
            untagged_vlans = options.fetch(:untagged_vlans, [])

            port_resources["ioa_interface"] ||= {}
            config = port_resources["ioa_interface"][interface] = {}

            # Teardown sets ports back to a "default" state. We want to ensure
            # that default config doesn't get squashed by portchannel specific config
            if !portchannel || options[:action] == :remove || smux?
              config["vlan_tagged"] = tagged_vlans.join(",") if !tagged_vlans.empty? || options[:action] == :remove
              config["vlan_untagged"] = untagged_vlans.join(",") unless untagged_vlans.empty?
              config["switchport"] = "true"
              config["portmode"] = "hybrid"
            else
              config["portchannel"] = portchannel
            end

            config["shutdown"] = "false"
            config["mtu"] = provider.mtu
            config["require"] = sequence if sequence

            @sequence = "Ioa_interface[%s]" % interface
            nil
          end

          def populate_interface_resource(name, properties)
            if iom_mode == "full-switch"
              super
            else
              ioa_interface_resource(name,
                                     :portchannel => properties[:portchannel],
                                     :tagged_vlans => properties[:tagged],
                                     :untagged_vlans => properties[:untagged],
                                     :action => properties[:action])
            end
          end

          # Creates vlt resources
          #
          # Creates a resources as hash normally parameters taken from vlt_data
          #
          # @note It will just sent as hash to to_puppet method, Run by puppet from the Yaml file
          # @param interface [String] port interface to assigned to a port channel
          # @param destination_ip [String] ip_address to set for creating back-up link
          # @param port_channel [String] port-channel to create up-link
          # @param unit_id [String] priority for th back-up link
          # @param mode [String] mode [pmux, vlt, fullswitch] configure in iom
          # @return [Hash] a resource for puppet run
          def vlt_ioa_mode_resource(interface, port_channel, destination_ip, unit_id, mode)
            {
              mode => {
                "iom_mode" => mode,
                "ioa_ethernet_mode" => "true",
                "ensure" => "present",
                "port_channel" => port_channel,
                "destination_ip" => destination_ip,
                "unit_id" => unit_id.to_s,
                "interface" => interface.to_s
              }
            }
          end

          def pmux_ioa_mode_resource(ethernet_mode, mode)
            {
              mode => {
                "iom_mode" => mode,
                "ensure" => "present",
                "ioa_ethernet_mode" => ethernet_mode.to_s,
                "vlt" => false
              }
            }
          end

          # Creates interfaces resources for consumption by puppet
          #
          # @return [Hash]
          def to_puppet
            %w(ioa_interface ioa_portchannel force10_interface force10_portchannel).each do |r_source|
              next unless port_resources[r_source]
              port_resources[r_source].keys.each do |name|
                resource = port_resources[r_source][name]

                ["vlan_tagged", "vlan_untagged", "tagged_vlan", "untagged_vlan"].each do |prop|
                  next unless resource[prop].is_a?(Array)
                  resource[prop] = resource[prop].sort.uniq.join(",")
                end
              end
            end

            port_resources
          end

          # Prepares the final internal state for a given action
          #
          # This should be called before {#to_puppet} to construct
          # the state based on interface information created using
          # {#configure_interface_vlan}
          #
          # The return value indicates if there were any interfaces to
          # configure for the action, if it's false there's no point
          # in calling process for the type as there's nothing to process
          #
          # @param [:add, :remove] action
          # @return [Boolean] if any interfaces were found for the action
          def prepare(action)
            reset!
            validate_vlans!
            populate_port_resources(action)
            !port_resources.empty?
          end

          # Find all the interfaces for a certain action and create VLAN resources
          #
          # Interfaces are made using {#configure_interface_vlan} and are
          # marked  as add or remove.  This finds all previously made interfaces
          # for a given action and calls {#vlan_resource) for each to construct
          # the correct asm::mxl resources
          #
          # @param [:add, :remove] action
          # @return [void]
          def populate_vlan_resources(action)
            action_interfaces = interface_map.select {|i| i[:action] == action}
            vlan_info = {}
            action_interfaces.each do |i|
              vlan_info[i[:vlan]] = i
            end

            vlan_info.each do |vlan, props|
              vlan_resource(vlan, props) unless action == :remove
            end
          end

          def disable_autolag
            port_resources["ioa_autolag"] = {
              "ioa_autolag" => {
                "ensure" => "absent"
              }
            }
          end

          # Reset each port in the switch to a default state
          #
          # @note this will populate the resources without running puppet
          # @return [void]
          def initialize_ports!
            if provider.model =~ /Aggregator/
              port_names.each do |port|
                configure_interface_vlan(port, "1", false, true)
                configure_interface_vlan(port, "1", true, true)
              end

              populate_port_resources(:remove)
            end
          end

          # Produce a list of port names for a certain switch
          #
          # @note only returned TE names not FC and only for a single unit
          # @return [Array<String>] array of port names
          def port_names
            (1..port_count).map {|i| "Te 0/%s" % i}
          end
        end
      end
    end
  end
end
