require "asm/private_util"
require "asm/cipher"

module ASM
  class Provider
    class Volume
      # Evaluate if we need same or separate classes for iSCSi, FC and FCoE implementation
      class Vnx < Provider::Base
        puppet_type "asm::volume::vnx"

        property :size,         :default => "100g",         :validation => /^\d+(k|m|t|KB|MB|GB|TB)$/i
        property :pool,         :default => "Pool 0",       :validation => String
        property :type,         :default => "nonthin",      :validation => String
        property :ensure,       :default => "present",      :validation => ["present", "absent"]
        property :folder,       :default => "asm",          :validation => String
        property :host_name,    :default => "",             :validation => String
        property :sgname,       :default => "",             :validation => String

        property :configuresan, :default => false,          :validation => :boolean, :tag => :extra
        property :porttype,     :default => "FibreChannel", :validation => String, :tag => :extra
        property :add_to_sdrs,  :default => "",             :validation => String, :tag => :extra

        def json_facts
          super + [
            "controllers_data",
            "disk_info_data",
            "disk_pools_data",
            "hbainfo_data",
            "pool_list",
            "pools_data",
            "raid_groups_data",
            "softwares_data",
            "storage_groups",
            "initiators"
          ]
        end

        def additional_resources
          vnx_server_initiators
        end

        # Gets the controllers from the storage facts
        #
        # Controllers contains information about wwpn in Hba_info
        #
        # @return [Array]
        # @raise [StandardError] when no related controllers_data are present
        def controllers
          facts["controllers_data"]["controllers"] || raise("No controllers found for %s" % type.puppet_certname)
        end

        # Checks if wwpn exists in storage facts
        #
        # Each controller contains HBA info that has ports that are connected to storage
        # tries to match the wwpn in HBAinfo
        #
        # @param wwpn [String] wwpn port value for the port
        # @return [String]
        def has_wwpn?(wwpn)
          logger.debug("Checking if wwpn %s is known on the HBA of %s" % [wwpn, type.puppet_certname])

          controllers.map { |c| return true if c["HBAInfo"] && c["HBAInfo"].include?(wwpn.upcase) }

          false
        end

        def fc?
          porttype == "FibreChannel" && configuresan
        end

        # Finds the device_alias for the storage
        #
        # Storage Alias should be configured manually in the brocade for the wwpn ports
        # tries to match with the emc storage ports and brocade_facts that contain wwpn to get device alias
        #
        # @note gets the first device_alias if there are multiple names to the same wwpn
        # @param switch [Type::Switch] facts data of brocade switch
        # @return [String]
        # @raise [StandardError] when no ports are found in the switch
        def fault_domain(switch)
          logger.debug("Trying to find ports that are related to volume %s on switch %s" % [type.puppet_certname, switch.puppet_certname])

          storage_ports = switch.nameserver_info.select do |_, info|
            info["port_info"].match(/CLARiiON.*SP\w.*FC/) && has_wwpn?(info["remote_wwpn"])
          end

          logger.debug("Found %d storage_ports entries for %s on switch %s" % [storage_ports.size, type.puppet_certname, switch.puppet_certname])

          if storage_ports.empty?
            raise("No storage alias found on switch %s for volume %s" % [switch.puppet_certname, type.puppet_certname])
          end

          storage_ports.sort_by { |_, v| v[:device_allias] }.first[1]["device_alias"].strip
        end

        def datastore_require(server)
          ["Asm::Fcdatastore[%s:%s]" % [server.lookup_hostname, uuid]]
        end

        def remove_server_from_volume!(server)
          server_object = vnx_server_resource(server)
          initiator_access_teardown = vnx_server_initiator(server)

          unless initiator_access_teardown.empty?
            server_object.merge!(initiator_access_teardown)
          end

          type.process_generic(type.puppet_certname, server_object, "apply", true, nil, type.guid)
        end

        # Remove server information from storage-group
        #
        # For deployment teardown case, delete the storage-group too.
        # For deleting storage group run a separate process_generic as storage-group cannot be deleted if there
        # is any existing host or volume mapped to the storage group
        # Also need additional delay in two operations. To take care of this we are running two separate process_generic operations
        def clean_access_rights!
          resources = {}
          type.related_servers.select(&:teardown?).each do |server|
            logger.warn("Cleaning access rights for server %s from volume %s" % [server.puppet_certname, type.puppet_certname])
            resource = vnx_server_resource(server)
            resources["asm::volume::vnx"] ||= {}
            resources["asm::volume::vnx"].merge!(resource["asm::volume::vnx"])

            initiator_cleanup_hash = vnx_server_initiator(server)
            unless initiator_cleanup_hash.empty?
              resources.merge!(initiator_cleanup_hash)
            end
          end

          unless resources.empty?
            type.process_generic(type.puppet_certname, resources, "apply", true, nil, type.guid)
          end

          if type.service.teardown?
            type.process_generic(type.puppet_certname, vnx_server_resource, "apply", true, nil, type.guid)
          end
        end

        # @todo port from ServiceDeployment
        def vnx_storage_group_info
          ASM::PrivateUtil.get_vnx_storage_group_info(type.puppet_certname)
        end

        # @todo port from ServiceDeployment
        def vnx_lun_id
          ASM::PrivateUtil.get_vnx_lun_id(type.puppet_certname, type.name, logger)
        end

        # @todo port from ServiceDeployment
        def host_lun_info(server)
          type.deployment.host_lun_info("ASM-#{server.deployment.id}", vnx_storage_group_info, vnx_lun_id)
        end

        def esx_datastore(server, _cluster, volume_ensure)
          datastore_name = "%s:%s" % [server.lookup_hostname, @uuid]

          unless debug?
            host_lun_number = host_lun_info(server)
            raise("Unable to find VNX LUN #{type.name} on the storage ") unless host_lun_number
          end

          {"esx_datastore" =>
            {datastore_name =>
              {"ensure" => volume_ensure,
               "datastore" => @uuid,
               "type" => "vmfs",
               "lun" => host_lun_number,
               "transport" => "Transport[vcenter]"
              }
            }
          }
        end

        def vnx_server_resource(server=nil)
          sg_name = "ASM-#{type.deployment.id}"

          resource_hash = {"asm::volume::vnx" =>
            {sg_name =>
              {"ensure"       => "absent",
               "sgname"       => sg_name
              }
            }
          }
          resource_hash["asm::volume::vnx"][sg_name]["host_name"] = server.hostname if server
          resource_hash
        end

        # Returns all of the server initiators
        #
        # @return [Hash]
        def vnx_server_initiators
          conflicting_initiator_hash = initiator_resource_hash(true)
          initiator_hash = initiator_resource_hash

          if !conflicting_initiator_hash.empty?
            require_keys = conflicting_initiator_hash.keys.collect { |x| conflicting_initiator_hash[x].keys}.flatten
            require_info = require_keys.collect { |x| "Vnx_initiator[#{x}]" }
            initiator_hash["vnx_initiator"].keys.map { |x| initiator_hash["vnx_initiator"][x]["require"] = require_info }
            Util.deep_merge!(initiator_hash, conflicting_initiator_hash)
          else
            initiator_hash
          end
        end

        # Returns resource hash for creating / updating initiator configuration on EMC VNX
        #
        # For EMC VNX, HBA UID (combination of WWNN and WWPN) is registered when FC connection is established
        # ESXi hostname gets registered automatically and for other OS information needs to be registered manually
        # In case server's HBA UID is already registered with another hostname (conflicting hostname), then ASM configured OS hostname
        # will not appear on EMC Host list. These conflicting entries needs to be removed
        #
        # Example: ASM Hostname - 'server1' and server's GUID is "11:11:11:11:11:11:22:22:22:22:22:22"
        # On VNX GUID "11:11:11:11:11:11:22:22:22:22:22:22" is registered with name "server2",
        # We first need to remove initiator entry for "server2" and then add new entry for "server1"
        #
        # For HyperV hostname is not registered automatically with the VNX. Initiators needs to be updated with
        # correct hostname so that host-list information is updated, which is further used for creating the storage groups
        #
        # @param conflict [Boolean], default value is false. when true return resources where HBA stored hostname differs from the server hostname"
        # @return [Hash]
        def initiator_resource_hash(conflict=false)
          type.related_servers.inject({}) do |resources, server|
            if conflict
              Util.deep_merge!(resources, vnx_server_initiator(server, true))
            else
              Util.deep_merge!(resources, vnx_server_initiator(server))
            end
            resources
          end
        end

        # Returns resource hash for configuring initiators required for HyperV and ESXi servers
        #
        # Returns resource hash for adding new initiator information when flag "conflict" is false
        # Returns resource hash for removing conflicting hostname when flag of "conflict" is true
        #
        # @param server [ASM::Type::Server]
        # @param conflict [Boolean]
        # @return [Hash]
        def vnx_server_initiator(server, conflict=false)
          resource_hash = {}
          return resource_hash if server.teardown? && conflict

          10.times do
            initiators = facts["initiators"]
            server.wwpn_wwnn.each_with_index do |val, index|
              hba_uid = "%s:%s" % [val[0], val[1]]

              if conflict
                host_initiators = initiators.find_all { |x| x["hba_uid"] == hba_uid}
                next if host_initiators.find_all { |x| x["hostname"] != server.hostname}.empty?
              elsif !conflict && vnx_ports(initiators, hba_uid).empty? && !type.teardown?
                logger.debug("No VNX ports found for %s while configuring volume %s, skipping" % [server, type.puppet_certname])
                next
              end

              resource_title = conflict ? "#{server.hostname}_#{index}_cleanup" : "#{server.hostname}_#{index}"
              resource_hash["vnx_initiator"] ||= {}
              resource_hash["vnx_initiator"][resource_title] = {
                "ensure" => (conflict || server.teardown?) ? "absent" : "present",
                "hba_uid" => hba_uid,
                "hostname" => server.hostname,
                "ports" => vnx_ports(initiators, hba_uid),
                "transport" => "Transport[vnx]"
              }
            end

            break if !resource_hash.empty? || server.teardown? || conflict

            type.update_inventory
            sleep(60)
          end

          if resource_hash.empty? && !server.teardown? && !conflict
            raise("Failed to find initiator information for server %s " % [server.puppet_certname])
          end

          resource_hash
        end

        # Returns VNX port for a specific server where initiator needs to be created
        #
        # @param initiators [Hash] Initiator information retrieved from VNX facts
        # @param hba_uid [String] HBA UID (WWNN and WWNN) combination of the server
        # @return [Hash]
        def vnx_ports(initiators, hba_uid)
          # TODO: This is a hack until we properly stub out storage in debug_service rake task
          return [{"sp" => "a", "sp_port" => 0}, {"sp" => "b", "sp_port" => 1}] if type.debug?

          initiator = initiators.find { |x| x["hba_uid"] == hba_uid}
          return [] unless initiator
          initiator["ports"].each { |x| x.delete("storage_group_name") }
          initiator["ports"]
        end

        def fc_datastore(_, _)
          nil
        end
      end
    end
  end
end
