#!/bin/env ruby

$LOAD_PATH.unshift File.expand_path(File.join(File.dirname(__FILE__), "..", "..", "lib"))

require "mocha/api"
require "json"
require "pry"
require "logger/colors"
require "asm/service"
require "asm/type/base"
require "rspec/expectations"
require "rspec/support"
require "optparse"

$recorded_resources = []

class ASM::Type::Base
  def process_generic(cert_name, config, puppet_run_type, override=true, server_cert_name=nil, asm_guid=nil, perform_inventory_update=false)
    $recorded_resources << {cert_name => config}
  end
end

def logger
  logger = Logger.new(STDOUT)
  logger.level = @debug ? Logger::DEBUG : Logger::WARN
  logger.formatter = ASM::LoggerFormat.new
  logger
end

def color(code)
  colors = {
    :red => "[31m",
    :green => "[32m",
    :yellow => "[33m",
    :cyan => "[36m",
    :bold => "[1m",
    :reset => "[0m"
  }

  colors[code] || ""
end

def colorize(code, msg)
  "%s%s%s" % [ color(code), msg, color(:reset) ]
end

def hash_dig(hash, keys, default=nil, sep="|")
  ans = keys.split(sep).inject(hash) do |memo, key|
    memo[key] if memo
  end

  ans.nil? ? default : ans
end

def find_fixtures(directory, pattern)
  Dir.glob(File.join(directory, "*.json")).grep(pattern).each do |fixture|
    next if File.basename(fixture) == "switch_inventory.json"
    if File.basename(fixture) =~ /(.+)_(inventory|device_config|facts|fc_interfaces).json/
      yield(fixture, $1)
    end
  end
end

def compact_switch_inventory!(inventory, directory)
  fixtures = []

  find_fixtures(directory, /device_config/) do |fixture|
    fixture = File.basename(fixture)
    if fixture =~ /(.+?)_device_config.json/
      fixtures << $1
    end
  end

  inventory.delete_if do |switch|
    if !fixtures.include?(switch["refId"])
      true
    else
      false
    end
  end
end

def create_captured_service(directory, test)
  switch_inventory_json = File.join(directory, "switch_inventory.json")
  deployment_json = File.join(directory, "deployment.json")
  asm_networks_json = File.join(directory, "asm_networks.json")
  raw_deployment = JSON.parse(File.read(deployment_json), :max_nesting => 100)
  raw_switches = JSON.parse(File.read(switch_inventory_json), :max_nesting => 100)

  compact_switch_inventory!(raw_switches, directory)

  ASM.logger = logger
  ASM.init_data_and_mutexes
  ASM::DeviceManagement.stubs(:run_puppet_device!)

  puppetdb = stub(:replace_facts_blocking! => nil)
  db = stub(:log => nil, :execution_id => raw_deployment["id"])

  deployment = stub(:id => raw_deployment["id"], :debug? => true, :process_generic => false, :puppetdb => puppetdb, :logger => logger, :db => db)
  service = ASM::Service.new(raw_deployment, :deployment => deployment)

  switch_collection = service.switch_collection
  switch_collection.stubs(:managed_inventory).returns(raw_switches)
  switch_collection.each {|s| s.stubs(:update_inventory!)}
  ASM::Service::SwitchCollection.stubs(:new).returns(switch_collection)

  service.stubs(:switch_collection).returns(switch_collection)

  if File.readable?(asm_networks_json)
    ASM::PrivateUtil.stubs(:get_network_info).returns(JSON.parse(File.read(asm_networks_json)))
  else
    logger.warn("No ASM Networks stub found in %s" % asm_networks_json)
  end

  find_fixtures(directory, /inventory/) do |fixture, certname|
    data = JSON.parse(File.read(fixture))
    ASM::PrivateUtil.stubs(:fetch_server_inventory).with(certname).returns(data)
    ASM::PrivateUtil.stubs(:fetch_managed_device_inventory).with(certname).returns([data])
  end

  find_fixtures(directory, /device_config/) do |fixture, certname|
    data = Hashie::Mash.new(JSON.parse(File.read(fixture)))
    ASM::DeviceManagement.stubs(:parse_device_config).with(certname).returns(data)
  end

  find_fixtures(directory, /facts/) do |fixture, certname|
    data = JSON.parse(File.read(fixture))
    ASM::PrivateUtil.stubs(:facts_find).with(certname).returns(data)
  end

  servers = service.resources_by_type("SERVER").each do |server|
    fixture = File.join(directory, "%s_network_config.json" % server.puppet_certname)
    network_config = ASM::NetworkConfiguration.new(JSON.parse(File.read(fixture)))
    network_config.instance_variable_set("@network_config_add_nic", true)
    server.stubs(:network_config).returns(network_config)
    server.stubs(:enable_switch_inventory!).returns(network_config)
    server.stubs(:wwpn_wwnn).returns([["testwwpn", "testwwnns"]])

    # unfortunately mocha doesnt let me pass a block else I'd have liked to log something here
    server.stubs(:power_off!)
    server.stubs(:power_on!)

    fixture = File.join(directory, "%s_fc_interfaces.json" % server.puppet_certname)
    if File.exist?(fixture)
      fc_views = JSON.parse(File.read(fixture), :symbolize_names => true)
      server.provider.stubs(:fc_views).returns(fc_views)
    else
      server.provider.stubs(:fc_views).returns([])
    end

    fixture = File.join(directory, "%s_fcoe_wwpns.json" % server.puppet_certname)
    if File.exist?(fixture)
      fcoe_wwpns = JSON.parse(File.read(fixture))
      ASM::WsMan.stubs(:get_fcoe_wwpn).with(server.device_config, anything).returns(fcoe_wwpns)
    else
      server.provider.stubs(:fcoe_wwpns).returns([])
    end

    default_servers_os_installed = hash_dig(test, "servers|all|servers_os_installed", false)
    default_servers_already_deployed = hash_dig(test, "servers|all|servers_already_deployed", false)

    server.stubs(:os_installed?).returns(
      hash_dig(test, "servers|#{server.puppet_certname}|servers_os_installed", default_servers_os_installed)
    )

    server.stubs(:deployment_completed?).returns(
      hash_dig(test, "servers|#{server.puppet_certname}|servers_already_deployed", default_servers_already_deployed)
    )
  end

  processor = service.create_processor("rules")
  processor.deployment = deployment

  service.resources.each {|r| r.deployment = deployment}

  [deployment, service]
end

def run_test(dir)
  test_json = File.join(dir, "test.json")
  test = JSON.parse(File.read(test_json))

  unless test["expected"]
    abort(colorize(:red, "Deployment %s has no expected data in its test.json" % dir))
  end

  expected = test["expected"]

  puts(colorize(:bold, ">>>>> %s in %s" % [test.fetch("scenario", "Untitled test"), dir]))
  puts
  puts

  deployment, service = create_captured_service(dir, test)

  service.switch_collection.configure_server_networking!

  unless expected == $recorded_resources
    puts "Expected:"
    pp expected
    puts
    puts "Got:"
    pp $recorded_resources
    puts
    puts "Diff:"
    differ = RSpec::Support::Differ.new(:color => true)
    puts differ.diff(expected, $recorded_resources)
    puts(colorize(:red, ">>>>> %s FAILED" % [test.fetch("scenario", "Untitled test")]))
    puts JSON.pretty_generate($recorded_resources) if @json
    exit(1)
  else
    if @debug
      puts "Got:"
      pp $recorded_resources
      puts
    end
    puts JSON.pretty_generate($recorded_resources) if @json
    puts(colorize(:green, ">>>>> %s PASSED" % [test.fetch("scenario", "Untitled test")]))
  end
end

opt = OptionParser.new

opt.on("--deployment DEPLOYMENT", "Directory with a captured deployment to test") do |v|
  @deployment = v
end

opt.on("--debug", "Directory with a captured deployment to test") do
  @debug = true
end

opt.on("--json", "Dump the calculated resources as JSON") do
  @json = true
end

opt.parse!

abort(colorize(:red, "Please specify a deployment with --deployment, see --help")) unless @deployment
abort(colorize(:red, "Deployment %s is not a directory" % @deployment)) unless File.directory?(@deployment)
abort(colorize(:red, "Deployment %s has no test.json" % @deployment)) unless File.readable?(File.join(@deployment, "test.json"))

include Mocha::API

run_test(@deployment)
