require 'spec_helper'
require 'asm/provider/switch/force10/blade_ioa'

describe ASM::Provider::Switch::Force10::BladeIoa do
  let(:logger) { stub(:debug => nil, :warn => nil, :info => nil) }
  let(:provider) { stub(:type => stub(:puppet_certname => "rspec"), :logger => logger, :mtu => "9216", :facts => {"iom_mode" => "programmable-mux"})}
  let(:blade) { ASM::Provider::Switch::Force10::BladeIoa.new(provider) }
  let(:vltdata) { {"uplinkId" => "vlt", "uplinkName" => "VLT", "portChannel" => nil, "portMembers" => ["Te 0/44", "Te 0/37"], "portNetworks" => []} }
  let(:vltdata2) { {"uplinkId" => "vlt", "uplinkName" => "VLT", "portChannel" => nil, "portMembers" => ["Te 0/44", "Te 0/37"], "portNetworks" => [], "model" => "PE-FN-410S-IOM"} }

  describe "#portchannel_resource" do
    it "should be a noop in smux mode" do
      blade.expects(:smux?).returns(true)
      blade.portchannel_resource(1)
      expect(blade.port_resources).to be_empty
    end

    it "should produce a valid resource" do
      blade.expects(:smux?).returns(false)
      blade.portchannel_resource(1, :tagged => %w{1 2 3}, :untagged => %w{4 5 6})
      expect(blade.port_resources).to eq(
        "ioa_portchannel" => {
          "1" => {
            "switchport" => "true",
            "portmode" => "hybrid",
            "shutdown" => "false",
            "ungroup" => "true",
            "mtu" => "9216",
            "vltpeer" => "true",
            "tagged_vlan" => %w{1 2 3},
            "untagged_vlan" => %w{4 5 6},
            "ensure" => "present"
          }
        }
      )
    end

    it "should support empty vlans" do
      blade.expects(:smux?).returns(false)
      blade.portchannel_resource(1)
      expect(blade.port_resources).to eq(
        "ioa_portchannel" => {
          "1" => {
            "switchport" => "true",
            "portmode" => "hybrid",
            "shutdown" => "false",
            "ungroup" => "true",
            "mtu" => "9216",
            "vltpeer" => "true",
            "ensure" => "present"
          }
        }
      )
    end

    it "should generate a valid resource for teardown" do
      blade.expects(:smux?).returns(false)
      blade.portchannel_resource(1, :tagged => %w{1 2 3}, :untagged => %w{4 5 6}, :action => :remove)
      expect(blade.port_resources).to eq(
        "ioa_portchannel" => {
          "1" => {
            "switchport" => "true",
            "portmode" => "hybrid",
            "shutdown" => "false",
            "ungroup" => "true",
            "mtu" => "9216",
            "vltpeer" => "true",
            "tagged_vlan" => %w{1 2 3},
            "untagged_vlan" => %w{4 5 6},
            "ensure" => "absent"
          }
        }
      )
    end

    it "should use force10_portchannel resource for full-switch mode" do
      blade.stubs(:iom_mode).returns("full-switch")
      blade.portchannel_resource(10)
      expect(blade.port_resources["force10_portchannel"]["10"]).to eq({
        "ensure" => "present",
        "switchport" => "true",
        "shutdown" => "false",
        "mtu" => "9216",
        "edge_port" => "pvst,mstp,rstp",
        "portfast" => "portfast",
        "portmode" => "hybrid",
        "edge_port" => "pvst,mstp,rstp",
        "vltpeer" => "false",
        "ungroup" => "false",
        "untagged_vlan" => [],
        "tagged_vlan" => []
      })
      expect(blade.port_resources["ioa_portchannel"]).to be(nil)
    end
  end

  describe "#initialize_ports!" do
    it "should not initialize IOAs" do
      provider.stubs(:model).returns("IOA")
      blade.expects(:port_names).never
      blade.initialize_ports!
    end

    it "should initialize each port" do
      provider.stubs(:model).returns("I/O-Aggregator")
      blade.expects(:port_names).returns(["Te 0/0", "Te 0/1"])
      blade.expects(:configure_interface_vlan).with("Te 0/0", "1", false, true)
      blade.expects(:configure_interface_vlan).with("Te 0/0", "1", true, true)
      blade.expects(:configure_interface_vlan).with("Te 0/1", "1", false, true)
      blade.expects(:configure_interface_vlan).with("Te 0/1", "1", true, true)
      blade.expects(:populate_port_resources).with(:remove)
      blade.initialize_ports!
    end
  end

  describe "#port_names" do
    it "should correctly generate Te names" do
      blade.expects(:port_count).returns(32)
      names = blade.port_names
      expect(names.size).to be(32)
      names.each_with_index do |name, idx|
        expect(name).to eq("Te 0/%s" % (idx + 1))
      end
    end
  end

  describe "#ioa_interface_resource" do
    it "should not double manage a resource" do
      blade.ioa_interface_resource("po 10", :tagged_vlans => [], :untagged_vlans => [])
      expect {
        blade.ioa_interface_resource("po 10", :tagged_vlans => [], :untagged_vlans => [])
      }.to raise_error("Ioa_interface[po 10] is already being managed on rspec")
    end

    it "should create a resource" do
      blade.ioa_interface_resource("po 10", :tagged_vlans => ["10", "11"], :untagged_vlans => ["12", "13"])
      blade.ioa_interface_resource("po 11", :tagged_vlans => [], :untagged_vlans => [])

      expect(blade.port_resources["ioa_interface"]["po 10"]).to eq({
        "vlan_tagged" => "10,11",
        "vlan_untagged" => "12,13",
        "switchport" => "true",
        "portmode" => "hybrid",
        "shutdown" => "false",
        "mtu" => "9216"
      })

      expect(blade.port_resources["ioa_interface"]["po 11"]).to eq({
        "switchport" => "true",
        "portmode" => "hybrid",
        "mtu" => "9216",
        "shutdown" => "false",
        "require" => "Ioa_interface[po 10]"
      })
    end

    it "should support portchnanels when not in smux mode" do
      blade.expects(:smux?).returns(false)
      blade.ioa_interface_resource("po 10", :tagged_vlans => ["10", "11"], :untagged_vlans => ["12", "13"], :portchannel => 10)
      expect(blade.port_resources["ioa_interface"]["po 10"]).to eq({
        "portchannel" => 10,
        "shutdown" => "false",
        "mtu" => "9216"
      })
    end

    it "should support portchnanels when in smux mode" do
      blade.expects(:smux?).returns(true)
      blade.ioa_interface_resource("po 10", :tagged_vlans => ["10", "11"], :untagged_vlans => ["12", "13"], :portchannel => 10)
      expect(blade.port_resources["ioa_interface"]["po 10"]).to eq({
        "vlan_tagged" => "10,11",
        "vlan_untagged" => "12,13",
        "switchport" => "true",
        "portmode" => "hybrid",
        "shutdown" => "false",
        "mtu" => "9216"
      })
    end

    it "should support adding empty vlans when servcer tear_down" do
       expect(blade.ioa_interface_resource("po 10", :tagged_vlans => [], :untagged_vlans => ["12", "13"], :portchannel => 10, :action => :remove))
       expect(blade.port_resources["ioa_interface"]["po 10"]).to eq({
                                                                        "vlan_tagged" => "",
                                                                        "vlan_untagged" => "12,13",
                                                                        "switchport" => "true",
                                                                        "portmode" => "hybrid",
                                                                        "shutdown" => "false",
                                                                        "mtu" => "9216"
                                                                    })

       end

    it "should not include tagged vlan if deployment has empty tagged vlans" do
      blade.expects(:smux?).returns(true)
      blade.ioa_interface_resource("po 10", :tagged_vlans => [], :untagged_vlans => ["12", "13"], :portchannel => 10)
      expect(blade.port_resources["ioa_interface"]["po 10"]).to eq({
                                                                       "vlan_untagged" => "12,13",
                                                                       "switchport" => "true",
                                                                       "portmode" => "hybrid",
                                                                       "shutdown" => "false",
                                                                       "mtu" => "9216"
                                                                   })
    end
  end

  describe "#configure_iom_mode!" do
    before(:each) do
      blade.stubs(:smux?).returns(false)
      blade.configure_interface_vlan("1", "1", false)
      blade.populate_port_resources(:add)
    end

    it "should support setting the sequence" do
      blade.configure_iom_mode!(false, true, vltdata)
      expect(blade.port_resources["ioa_mode"]["vlt"]["require"]).to eq("Ioa_interface[1]")
    end

    it "should supprot full_switch for FNIOA without vlt" do
      provider.type.stubs(:model).returns("PE-FN-410S-IOM")
      blade.configure_iom_mode!(true, true, nil)
      expect(blade.port_resources["ioa_mode"]).to include("fullswitch")
      expect(blade.sequence).to eq("Ioa_mode[fullswitch]")
    end

    it "should fail for multiple invocations" do
      blade.configure_iom_mode!(false, true, vltdata)
      expect { blade.configure_iom_mode!(true, false, vltdata) }.to raise_error("iom mode resource is already created in this instance for switch rspec")
    end

    it "should support fullswitch mode for FNIOA" do
      blade.configure_iom_mode!(false, true, vltdata2)
      expect(blade.port_resources["ioa_mode"]).to include("fullswitch")
      expect(blade.sequence).to eq("Ioa_mode[fullswitch]")
    end

    it "should support fullswitch mode without ethernet_mode for FNIOA" do
      blade.configure_iom_mode!(false, false, vltdata2)
      expect(blade.port_resources["ioa_mode"]).to include("fullswitch")
      expect(blade.port_resources["ioa_mode"]["fullswitch"]["ioa_ethernet_mode"]).to eq("true")
      expect(blade.sequence).to eq("Ioa_mode[fullswitch]")
    end

    it "should support vlt mode" do
      blade.configure_iom_mode!(false, true, vltdata)
      expect(blade.port_resources["ioa_mode"]).to include("vlt")
      expect(blade.sequence).to eq("Ioa_mode[vlt]")
    end

    it "should support pmux mode without ethernet_mode" do
      provider.type.stubs(:model).returns("I/O-Aggregator")
      blade.configure_iom_mode!(true, false)
      expect(blade.port_resources["ioa_mode"]).to include("pmux")
      expect(blade.port_resources["ioa_mode"]["pmux"]["ioa_ethernet_mode"]).to eq("false")
      expect(blade.sequence).to eq("Ioa_mode[pmux]")
    end

    it "should support pmux mode with ethernet_mode" do
      provider.type.stubs(:model).returns("I/O-Aggregator")
      blade.configure_iom_mode!(true, true)
      expect(blade.port_resources["ioa_mode"]).to include("pmux")
      expect(blade.port_resources["ioa_mode"]["pmux"]["ioa_ethernet_mode"]).to eq("true")
      expect(blade.port_resources["ioa_mode"]["pmux"]["vlt"]).to eq(false)
      expect(blade.sequence).to eq("Ioa_mode[pmux]")
    end

    it "should support both vlt and pmux being false" do
      blade.configure_iom_mode!(false, false)
      expect(blade.port_resources).to_not include("ioa_mode")
    end
  end

  describe "#validate_vlans!" do
    it "should allow multiple untagged vlans on different ports" do
      blade.configure_interface_vlan("1", "1", false)
      blade.configure_interface_vlan("2", "1", false)

      blade.validate_vlans!
    end

    it "should not allow multiple untagged vlans on the same port" do
      blade.configure_interface_vlan("1", "1", false)
      blade.configure_interface_vlan("1", "2", false)

      expect {
        blade.validate_vlans!
      }.to raise_error("can only have one untagged network but found multiple untagged vlan requests for the same port on rspec")
    end
  end

  describe "#configure_interface_vlan" do
    it "should add the interface correctly" do
      blade.configure_interface_vlan("1", "1", true)
      blade.configure_interface_vlan("2", "2", false)
      blade.configure_interface_vlan("3", "3", true, true)

      expect(blade.interface_map).to include({:interface => "1", :portchannel => nil, :vlan => "1", :tagged => true, :action => :add})
      expect(blade.interface_map).to include({:interface => "2", :portchannel => nil, :vlan => "2", :tagged => false, :action => :add})
      expect(blade.interface_map).to include({:interface => "3", :portchannel => nil, :vlan => "3", :tagged => true, :action => :remove})
    end
  end

  describe "#to_puppet" do
    it "should join array properties to strings" do
      blade.stubs(:port_resources).returns({
        "ioa_interface" => {
          "Te 1/1" => {
            "vlan_tagged" => ["1", "2", "3"],
            "vlan_untagged" => ["4", "5", "6"],
          }
        }
      })

      expect(blade.to_puppet).to eq({
        "ioa_interface" => {
          "Te 1/1" => {
            "vlan_tagged" => "1,2,3",
            "vlan_untagged" => "4,5,6",
          }
        }
      })
    end
  end

  describe "#prepare" do
    it "should correctly configure the internal state" do
      prepare = sequence(:prepare)
      blade.expects(:reset!).in_sequence(prepare)
      blade.expects(:validate_vlans!).in_sequence(prepare)
      blade.expects(:populate_port_resources).with(:add).in_sequence(prepare)

      blade.stubs(:port_resources).returns({})

      expect(blade.prepare(:add)).to be(false)
    end

    it "should correctly indicate if a process! is needed" do
      blade.stubs(:reset!)
      blade.stubs(:populate_interface_resources)

      blade.stubs(:port_resources).returns({})
      expect(blade.prepare(:add)).to be(false)

      blade.stubs(:port_resources).returns({"ioa_interface" => {}})
      expect(blade.prepare(:add)).to be(true)
    end
  end

  describe "#populate_interface_resource" do
    it "should use force10_interface resource for full-switch mode" do
      blade.stubs(:iom_mode).returns("full-switch")
      blade.populate_interface_resource("te 0/1", :action => :add, :untagged => [25], :tagged => [20])
      expect(blade.port_resources["force10_interface"]["te 0/1"]).to eq({
        "ensure" => "present",
        "untagged_vlan" => [25],
        "tagged_vlan" => [20],
        "shutdown" => "false",
        "mtu" => "9216",
        "protocol" => "lldp",
        "switchport" => "true",
        "portmode" => "hybrid",
        "portfast" => "portfast",
        "edge_port" => "pvst,mstp,rstp" })
      expect(blade.port_resources["ioa_interface"]).to eq(nil)
    end
  end
end
